﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace pr200_1
{
    // Объявление делегата: 
    delegate void MyDelegate(string t);

    // Первая структура: 
    struct Alpha {
        // Открытое текстовое поле: 
        public string name;
        // Закрытое поле, являющееся ссылкой 
        // на экземпляр делегата: 
        private MyDelegate myevent;
        // Объявление события: 
        public event MyDelegate MyEvent {
            // Аксессор для добавления ссылки на метод
            // в список обработчиков события:
            add {
                myevent += value;
            }
            // Аксессор для удаления ссылки на метод 
            // из списка обработчиков события: 
            remove {
                myevent -= value;
            }
        }
        // Конструктор: 
        public Alpha(string t) {
            // Значения полей:
            name = t;
            myevent = null;
        }
        // Метод для генерирования событий: 
        public void RaiseMyEvent() {
            // Если ссылка не пустая, то вызывается 
            // экземпляр делегата: 
            if (myevent != null) 
                myevent("Alpha: " + name);
        }
    }

    // Вторая структура: 
    struct Bravo {
        // Открытое текстовое поле: 
        public string name;
        // Объявление события: 
        public event MyDelegate MyEvent;
        // Метод для генерирования события: 
        public void RaiseMyEvent() {
            // Если список обработчиков не пустой, то 
            // генерируется событие: 
            if (MyEvent != null) 
                MyEvent("Bravo: " + name);
        }
    }

    // Класс с главным методом:
    class Program
    {
        // Статический метод: 
        static void show(string t) {
            Console.WriteLine("Произошло событие");
            Console.WriteLine(t);
            Console.WriteLine("------------------");
        }
        // Главный метод:
        static void Main()
        {
            // Создание экземпляра первой структуры:
            Alpha A = new Alpha("Экземпляр A");
            // Добавление ссылки на метод в список обработчиков 
            // события:
            A.MyEvent += show;
            // Генерирование события:
            A.RaiseMyEvent();
            // Создание экземпляра второй структуры:
            Bravo B = new Bravo();
            // Присваивание значения текстовому полю:
            B.name = "Экземпляр B";
            // Добавление ссылки на метод в список обработчиков 
            // события:
            B.MyEvent += show;
            // Генерирование событие:
            B.RaiseMyEvent();
            // Задержка:
            Console.ReadLine();
        }
    }
}
